/*
 *  StagingBuffer.h
 *  EDL
 *
 *  Created by Jerry on Wed Feb 18 2004.
 *  Copyright (c) 2004 Frontier Design Group. All rights reserved.
 *
 */

#ifndef __cSTAGINGBUFFER_CPP
#define __cSTAGINGBUFFER_CPP

#include "AudioEvent.h"

#define BUFFER_BYTES	(1024 * 128)
#define BUFFER_SAMPLES  (BUFFER_BYTES/8)
#define BYTES_PER_SAMPLE 8

#define LOW

#define kFADEIN		true
#define kFADEOUT	false

// base class for the staging buffers used to stage between files & 
// playback
//
class StagingBuffer {

public:
	StagingBuffer();
	~StagingBuffer();


private:
	// members
	QWORD			sampleStart;		// buffer's sample location in time
	QWORD			samples;			// number of samples in buffer
	QWORD*          memBuffer;			// where files transfer into/out of
	QWORD*			bufPtr;				// to manage data xfers
	bool			full;

public:
	QWORD   SamplesLeft()	    { return BUFFER_SAMPLES - samples;          }
	QWORD	Size()			    { return samples;	                        }
	QWORD	Start()			    { return sampleStart;                       }
	bool	Full()			    { return (full || samples==BUFFER_SAMPLES); }
	void	SetFull()		    { full  = true;			                    }
	bool    Contains(QWORD loc)	{ return (sampleStart <= loc && loc < (sampleStart + samples));	}
    void    Validate()          { bufPtr = memBuffer; samples = 0; full = false;       }

public:
					 
	QWORD			ZeroFill(QWORD samples, QWORD sampleLoc);
	QWORD			Render( char* lsrc, char* rsrc, QWORD sampleCnt, WORD width, ChanTypes type, QWORD sampleLoc );
	QWORD			Fade(QWORD fadeSampleStart, QWORD offset, bool fadeIn, char stride);
    QWORD*          Buffer() { return memBuffer; }

private:
	void			Render16bit( char*  source, char*  rsource, DWORD samples, ChanTypes type );
	void			Render24bit( char*  source, char*  rsource, DWORD samples, ChanTypes type );
	//void			Render32bit( char* source, char* rsource, DWORD samples, ChanTypes type );
};


/*
 * 
 *  Primarily responsible for moving audio data from filebufs
 *  and render into staging buffer
 *
 */
class   EDLRenderer  {

public:
	EDLRenderer();


private:
	bool            firstFill;                      // first fill since reset? (controls initial fadein)
	StagingBuffer   stageBuf;			            // 32bit staging buffer

	AudioEvent*     lastEvent;                      // last event we were working on
	QWORD			fadeInOffset;					// for tracking fade overlaps in buffers
	QWORD			fadeOutOffset;					//  ""    ""
public:
	void			Init();
    bool            Fill(EDL* edl, QWORD& currentSample, char stride);
    QWORD*          Buf() { return stageBuf.Buffer(); }

};


#endif  //__cSTAGINGBUFFER_CPP